-- Cannonball -- the Enigma Level
-- (c) 2003 Petr Machata/ant_39
-- Licensed under GPL v2.0 or above
-- 2003-01-08
-- 2003-01-10 -- modified base floor beneath doorE, doorF, removed document
-- 2003-01-11 -- modified so that it uses object groups from ant.lua
-- 2003-02-09 -- fixes in object groups

dofile(enigma.FindDataFile("levels/ant.lua"))

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
function construct(x, y)
   set_floor("fl-metal", x, y)
end

function destruct(x, y)
   set_floor("fl-abyss", x, y)
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
forcecount = 0
force = 100
act_state = 0
DOORCOUNT = 0

function forceup()
   forcecount = forcecount + 1
   enigma.AddConstantForce(force,0)
end

function forcedn()
   enigma.AddConstantForce((-1)*force*forcecount,0)
   forcecount = 0
end

function run_trigger_action()
   if (act_state == 0) then
      act_state = 1
   end
end

function restart_trigger_action()
   act_state = 0
   forcedn()
end

function tick()
   if (act_state == 0) then
      send_group_message(doors_A, "open", nil)
   elseif (act_state == 1) then
      send_group_message(doors_A, "close", nil)
      act_state = act_state + 1
   elseif (act_state == 2) then
      send_group_message(doors_B, "open", nil)
      act_state = act_state + 1
   elseif (act_state == 3) then
      forceup()
      act_state = act_state + 1
   elseif (act_state == 4) then
      send_group_message(doors_B, "close", nil)
      act_state = 0
   else
      act_state = 0
   end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

cells={}
cells[";"]=cell{floor={face="fl-tigris"}}
cells[":"]=cell{floor={face="fl-sahara"}}
cells["."]=cell{floor={face="fl-abyss"}}
cells["_"]=cell{floor={face="fl-metal"}}
cells["w"]=cell{floor={face="fl-wood"}}

cells["v"]=cell{floor={face="fl-gradient", attr={type=1}}}
cells["^"]=cell{floor={face="fl-gradient", attr={type=2}}}
cells[">"]=cell{floor={face="fl-gradient", attr={type=23}}}
cells["|"]=cell{floor={face="fl-gradient", attr={type=3}}}
cells["T"]=cell{floor={face="fl-gradient", attr={type=5}}}
cells["L"]=cell{floor={face="fl-gradient", attr={type=7}}}
cells["R"]=cell{floor={face="fl-gradient", attr={type=9}}}
cells["/"]=cell{floor={face="fl-gradient", attr={type=10}}}

cells[" "]=cell{parent={{checkerfloor,{cells[":"], cells[";"]}}}}

cells["="]=cell{parent=cells[" "],stone={face="st-rock5"}}
cells["#"]=cell{parent=cells[" "],stone={face="st-rock6"}}

cells["&"]=cell{parent=cells[" "],stone={face="st-oneway", attr={orientation=EAST}}}
cells["%"]=cell{parent=cells[" "],stone={face="st-oneway", attr={orientation=WEST}}}

cells["x"]=cell{parent=cells[" "],stone={face="st-grate1"}}
cells["e"]=cell{parent=cells["w"],stone={face="st-door", attr={name="doorE", type="v"}}}
cells["f"]=cell{parent=cells["w"],stone={face="st-door", attr={name="doorF", type="v"}}}

doors_A = {};
doors_B = {};
cells["?"]=cell{parent={cells[" "], {add_multistone, {"st-door", doors_A, {type="h"}}}}}
cells["!"]=cell{parent={cells[" "], {add_multistone, {"st-door_b", doors_B, {type="v"}}}}}

cells["D"]=cell{parent=cells["w"],stone={face="st-death"}}
cells["W"]=cell{parent=cells["w"],stone={face="st-brick"}}
cells["E"]=cell{parent=cells["w"],stone={face="st-floppy", attr={action="openclose", target="doorE"}}}
cells["F"]=cell{parent=cells["w"],stone={face="st-floppy", attr={action="openclose", target="doorF"}}}

cells["~"]=cell{parent={cells[" "]},stone={face="st-timer", attr={action="callback", target="tick", interval=1}}}
cells["`"]=cell{parent={cells[" "]},stone={face="st-timer", attr={action="callback", target="move_constructors", interval=0.15}}}

cells["+"]=cell{parent=cells[" "],item={face="it-trigger", attr={action="callback", target="forceup"}}}
cells["-"]=cell{parent=cells["_"],item={face="it-trigger", attr={action="callback", target="forcedn"}}}
cells["*"]=cell{parent={cells[">"], cells["+"]}}
cells["o"]=cell{parent=cells[" "],item={face="it-trigger", attr={action="callback", target="run_trigger_action"}}}
cells["9"]=cell{parent=cells[" "],item={face="it-trigger", attr={action="callback", target="restart_trigger_action", invisible=1}}}

cells["h"]=cell{parent=cells["w"],item={face="it-floppy"}}
cells["s"]=cell{parent=cells["x"],item={face="it-spring1"}}

cells["$"]=cell{parent={cells["."], {path_empty, {1}}                }}
cells["@"]=cell{parent={cells["$"], {path_train, {1}}, cells["_"]}}
cells["c"]=cell{parent={cells["@"], {path_constructor,{1, construct}}}}
cells["d"]=cell{parent={cells["$"], {path_destructor, {1, destruct}} }}
cells["X"]=cell{parent={cells["x"], cells["$"]}}
cells["Y"]=cell{parent={cells["x"], cells["@"]}}
cells["y"]=cell{parent={cells["x"], cells["w"]}}

cells["0"]=cell{parent=cells["w"],stone={face="st-oxyd", attr={flavor="b", color="0"}}}

cells["O"]=cell{parent=cells["9"],item={face="ac-blackball", attr={player=0}, actor=1}}

level = {
   "====================.....................................~WWWWWWWWWWWWWWWWWWW",
   "=xxxxxxxxxxxxxxxxxx==wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwWW......$$c@Y@@d$...W",
   "=xx        O       %ww=..............www..............Www%wwwwwwXwwwWwwwXwwwW",
   "=x 9               ====...............w...............WWWW...wwwXXXXXXXXXwwwW",
   "=x#?###############=..................w............WWWWWWW...wwwwwwwWwwwwwWWW",
   "=x# !TvvvvvvvvvvvR&__................www...........wwwwww&wwwwwwwwwwWEwwwwe0W",
   "=x#o!|>>*>>>*>>>*>&____________-.....whw.........wwwwwwwDWWWWWWWWWWWWWWWWWWWW",
   "=x# !L^^^^^^^^^^^/&__................www...........wwwwww&wwwwwwwwwwWFwwwwf0W",
   "=x#?###############=..................w............WWWWWWW...wwwwwwwWwwwwwWWW",
   "=x 9               ====...............w...............WWWW...wwwXXXXXXXXXwwwW",
   "=xx                %ww=..............www..............Www%wwwwwwXwwwWwwwXwwwW",
   "=sxxxxxxxxxxxxxxxxx==wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwWW......$$c@Y@@d$...W",
   "====================.....................................`WWWWWWWWWWWWWWWWWWW"
}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
randomseed(enigma.GetTicks())
create_world_by_map(level)
oxyd_shuffle()
